%SNAKE   Minimizza iterativamente lo snake sull'immagine I
%
% USO:
%  [xf yf i] = snake(I, x, y, alpha, beta, resolution, delta, iterazioni,T1,T2)
%
% PARAMETERI:
%  I: immagine da segmentare;
%  x,y: vettori di coordinate che identificano lo snake iniziale
%  alpha: coefficiente di tensione 
%  beta: coefficiente di rigidit
%  resolution: risoluzione temporale
%  delta: fattore di compensazione se l'immagine ha basso contrasto
%  iterazioni: numero di iterazioni da effettuare prima del controllo di
%              convergenza
%  T1,T2: soglie sulla posizione degli oggetti della depth map
%  xf,yf: vettori di coordinate che identificano lo snake finale
% NOTE:
%  L'immagine viene convertita in scala di grigi.
function [xf yf i]=snake(I,x,y,alpha,beta,resolution,delta,iterazioni,T1,T2)
[h w c]=size(I);
if(T1~=-1)
    % verifica delle soglie
    if(T1>T2)
        temp=T1;
        T1=T2;
        T2=temp;
    end
    % selezione parziale in base al livello di grigio
    I1=I;
    for i=1:size(I,1)
        for j=1:size(I,2)
            if(I(i,j)<T1-0.05)
                I1(i,j)=0;
            end
             if(I(i,j)>T2+0.05)
                I1(i,j)=0;
            end
        end
    end
else
    I1=I;
end
%Enhancement dell'immagine
I1=imadjust(I1);
%Sogliatura automatica
T=graythresh(I1);
%Padding per evitare i contorni sui bordi 
padded=padarray(I1,[h w],'replicate');
%Edge detection
BW = edge(padded,'canny',T);
BW=BW(h+1:2*h,w+1:2*w);
%Landscape energia esterna
distancemap=sqrt(bwdist(BW));
if(max(max(abs(distancemap)))==0)
    fprintf('Immagine non segmentabile perch omogenea.\n');
    return;
end
distancemap=im2double(imcomplement(distancemap/max(max(abs(distancemap)))));

%Peso i bordi trovati con canny rispetto al valore di grigio della regione
%di appartenenza per gestire le sovrapposizioni e il rumore
smoothed=imfilter(I1, fspecial('gaussian', [3 3], 1), 'replicate');
gradiente=distancemap+distancemap.*smoothed;

%Calcolo del gradiente sulle due direzioni
fprintf(' + calcolo della forza esterna...')
[FX,FY] = gradient(gradiente/max(max(gradiente)));

%Normalizzazione dei coefficienti
gamma = (100*delta/resolution)/max(max(FX));
alpha=alpha/(100*delta)*max(max(FX));
beta=beta/(100*delta)*max(max(FX));


%Costruzione della soluzione all'equazione di Eulero-Lagrange
N = length(x);
a = gamma*(2*alpha+6*beta)+1;
b = gamma*(-alpha-4*beta);
c = gamma*beta;
P = diag(repmat(a,1,N));
P = P + diag(repmat(b,1,N-1), 1) + diag(   b, -N+1);
P = P + diag(repmat(b,1,N-1),-1) + diag(   b,  N-1);
P = P + diag(repmat(c,1,N-2), 2) + diag([c,c],-N+2);
P = P + diag(repmat(c,1,N-2),-2) + diag([c,c], N-2);
P = inv(P);
fprintf('FATTO!\n')
%Deformazione snake
for i = 1:iterazioni*gamma*2
   
    % calcolo della forza esterna
    fex= interpn(double(FX),double(y),double(x),'linear'); 
    fey= interpn(double(FY),double(y),double(x),'linear');
    tempfex=fex;
    tempfey=fey;
    % spostamento dello snake
    tempx=x;
    tempy=y;
    x = P*(x+gamma*fex);
    y = P*(y+gamma*fey);
  
    % verifica out-of-bounds
    if(max(x)>w-1)
        ind=find(x>w-1);
        x(ind)=w-1;
    end
    if(max(y)>h-1)
        ind=find(y>h-1);
        y(ind)=h-1;
    end
      if(min(x)<1)
        ind=find(x<1);
        x(ind)=1;
    end
    if(min(y)<1)
        ind=find(y<1);
        y(ind)=1;
    end

    % plot del contorno
    if (mod(i,iterazioni/100)==0)
        hold on, plot([x;x(1)],[y;y(1)],'y')
        drawnow;
    end

    % verifica della convergenza
    if(i>iterazioni)
        % distanza media tra le due curve
        distmean=mean((sqrt((tempx-x).^2 + (tempy-y).^2)));
        distmax=max((sqrt((tempx-x).^2 + (tempy-y).^2)));
        if((distmean<0.0001*power(gamma,2)&&distmax<0.01*gamma)||distmean<0.001*gamma)
                break;
        end
    end
end
xf=x;
yf=y;